from cml.shared.parameter import PROTOCOL_LEVEL


__all__ = (
    "FeatureSelector",
)


def log_selection(func):
    def wrapper(self, learnblock):
        reduced_learnblock = func(self, learnblock)
        if self.logger and self.logger.level == PROTOCOL_LEVEL:
            self.logger.protocol(
                "\t".join([
                    "",
                    str(learnblock.length),
                    str(learnblock.min_timestamp),
                    str(learnblock.max_timestamp),
                    str(learnblock.relatives),
                    str(learnblock.n_cluster),
                    str(learnblock.feature_count),
                ])
            )
        return reduced_learnblock
    return wrapper


class FeatureSelector:

    def __init__(self, filter_method, embedded_method, settings):
        self.filter_method = filter_method
        self.embedded_method = embedded_method
        self.settings = settings
        self.logger = None

    @log_selection
    def select(self, learnblock):
        while 1:
            if self._to_many_features(learnblock.feature_count) \
                    or self.settings.max_model_reduction:
                past_feature_number = learnblock.feature_count

                if self._filter_method_criteria(learnblock.feature_count,
                                                learnblock.rows):
                    remove_features = self.filtering(learnblock)
                    learnblock.drop_columns_by_index(remove_features)
                else:
                    remove_features = self.embedding(learnblock)
                    learnblock.drop_columns_by_index(remove_features)

                if past_feature_number == learnblock.feature_count \
                        and not self._to_many_features(
                            learnblock.feature_count):
                    return learnblock

                elif past_feature_number == learnblock.feature_count \
                        and self._to_many_features(learnblock.feature_count):
                    raise ValueError("Can't reduce learnblock")
                else:
                    continue

            else:
                return learnblock

    def filtering(self, learnblock):
        trained_model = self.filter_method.train(learnblock)
        return trained_model.reduce(learnblock)

    def embedding(self, learnblock):
        trained_model = self.embedded_method.train(learnblock)
        return trained_model.reduce()

    def _to_many_features(self, feature_count):
        return feature_count > self.settings.max_features

    def _filter_method_criteria(self, feature_count, samples_count):
        return feature_count > self.settings.max_filter_x or \
                samples_count > self.settings.max_filter_y
