from os import mkdir
from os.path import expanduser, join
from datetime import datetime
from logging import getLogger, FileHandler, StreamHandler

from cml.shared.parameter import PROTOCOL_LEVEL
from cml.usecases.usecase import Usecase
from cml.domain import (
    Preprocessor,
    DataSource,
    LearnblockIdentifier,
    Constructor,
    FeatureSelector,
    Reconstructor
)


class PreprocessUsecase(Usecase):
    def process(self, request):
        preprocesser = Preprocessor(request.pre_settings)
        cleaned_data = preprocesser.clean(request.source_adapter)
        learnblock_identifier = LearnblockIdentifier(request.block_settings,
                                                     request.density_estimator,
                                                     request.relative_extrema)
        data_source = DataSource(cleaned_data,
                                 learnblock_identifier,
                                 block_size=request.block_settings.block_size)
        return data_source


class CreateConstructorUsecase(Usecase):
    def process(self, request):
        constructor = Constructor(request.ml_models, request.settings)
        constructor.construction_type = request.settings.construct_type
        return constructor


class FeatureSelectionUsecase(Usecase):
    def process(self, request):
        return FeatureSelector(request.filter_method,
                               request.embedded_method,
                               request.settings)


class ReconstructionUsecase(Usecase):
    def process(self, request):
        return Reconstructor()


class KnowledgeSearchUsecase(Usecase):
    def process(self, request):
        pass
